@@ -1,4 +1,4 @@
-;;; diff-ansi.el --- Display diff's using alternative diffing tools -*- lexical-binding: t -*-
+;;; diff-ansi.el --- Display diffs using alternative diffing tools -*- lexical-binding: t -*-
 
 ;; SPDX-License-Identifier: GPL-2.0-or-later
 ;; Copyright (C) 2021  Campbell Barton
@@ -13,7 +13,7 @@
 
 ;; Support showing color diffs using external tools.
 
-;;; Usage
+;;; Usage:
 
 ;; See readme.rst.
 ;;
@@ -43,14 +43,13 @@
 ;; Custom Variables
 
 (defgroup diff-ansi nil
-  "Configure smooth scrolling when jumping to new locations."
-  :group 'scrolling)
+  "Display diffs using external display commands."
+  :group 'diff)
 
 (defcustom diff-ansi-use-magit-revision-diff t
   "Use diff-ansi for `magit-revision-mode'."
   :type 'boolean)
 
-
 (defcustom diff-ansi-tool 'delta
   "Command to use for generating the diff."
   :type
@@ -60,8 +59,8 @@
 
            (symbol :tag "Use `diff-ansi-tool-custom' command." custom)))
 
-;; Note that this has it's values extracted and isn't used directly.
-(defface diff-ansi-default-face (list (list t :foreground "black" :background "black"))
+;; Note that this has its values extracted and isn't used directly.
+(defface diff-ansi-default-face (list (list t :foreground "white" :background "black"))
   "Face used to render black color."
   :group 'diff-ansi)
 
@@ -98,13 +97,13 @@ This must take the diff content as the `standard-input'."
 (defcustom diff-ansi-chunks-size 16384
   "Number of characters to process at once.
 Used for `progressive' and `multiprocess' methods."
-  :type 'integer)
+  :type 'natnum)
 
 (defcustom diff-ansi-multiprocess-jobs nil
   "Number of simultaneous jobs to launch when using `multiprocess' method.
 
-A nil value detects the number of processes on the system (when supported)."
-  :type 'integer)
+A nil value auto-detects the number of processors using `num-processors'."
+  :type '(choice (const :tag "Auto-detect" nil) (natnum :tag "Number of jobs")))
 
 (defcustom diff-ansi-verbose-progress nil
   "When enabled, show the progress of progressive conversion in the echo area.
@@ -155,8 +154,8 @@ It can be useful to show progress when viewing very large diffs."
 ;; ---------------------------------------------------------------------------
 ;; Forward Declarations
 
-;; Needed since the the `eval' causes the function to be hidden.
-(declare-function diff-ansi--ansi-color-apply-on-region-with-bg-impl "diff-ansi")
+;; Needed since the `eval' causes the function to be hidden.
+(declare-function diff-ansi--ansi-color-apply-on-region-with-bg-impl "ansi-color")
 (declare-function magit-wash-sequence "magit-section")
 
 
@@ -166,7 +165,7 @@ It can be useful to show progress when viewing very large diffs."
 (defmacro diff-ansi--with-advice (advice &rest body)
   "Execute BODY with ADVICE temporarily enabled.
 
-Advice are triplets of (SYMBOL HOW FUNCTION),
+Each advice is a triplet of (SYMBOL HOW FUNCTION),
 see `advice-add' documentation."
   (declare (indent 1))
   (let ((advice-list advice)
@@ -215,9 +214,9 @@ see `advice-add' documentation."
          (message omessage)))))
 
 (defmacro diff-ansi--with-temp-directory (name &rest body)
-  "Bind NAME to the name of a new temporary file and evaluate BODY.
+  "Bind NAME to the name of a new temporary directory and evaluate BODY.
 Delete the temporary file after BODY exits normally or non-locally.
-NAME will be bound to the file name of the temporary file.
+NAME will be bound to the file name of the temporary directory.
 
 The following keyword arguments are supported:
 
@@ -226,7 +225,7 @@ The following keyword arguments are supported:
 
 :suffix STRING
   If non-nil, pass STRING to `make-temp-file' as the SUFFIX argument."
-  (declare (indent 1) (debug (symbolp body)))
+  (declare (indent 1) (debug (sexp body)))
   (unless (symbolp name)
     (error "Expected name to be as symbol, found %S" (type-of name)))
   (let ((keyw nil)
@@ -243,11 +242,11 @@ The following keyword arguments are supported:
          (pop body))))
     (when extra-keywords
       (error "Invalid keywords: %s" (mapconcat #'symbol-name extra-keywords " ")))
-    (let ((temp-file (make-symbol "temp-file"))
-          (prefix (or prefix ""))
-          (suffix (or suffix "")))
+    (let ((temp-file (make-symbol "temp-file")))
+      (setq prefix (or prefix ""))
+      (setq suffix (or suffix ""))
       `(let* ((,temp-file (file-name-as-directory (make-temp-file ,prefix t ,suffix nil)))
-              (,name (file-name-as-directory ,temp-file)))
+              (,name ,temp-file))
          (unwind-protect
              (progn
                ,@body)
@@ -294,7 +293,7 @@ Additional keyword arguments may also be passed in.
                 (:input
                  (cond
                   ((null v)) ; No input.
-                  ((eq v t)) ; String input (standard input data).
+                  ((eq v t)) ; True for string input (standard input data).
                   ((stringp v))
                   ((bufferp v)
                    (unless (buffer-live-p v)
@@ -304,14 +303,14 @@ Additional keyword arguments may also be passed in.
                  (setq input v))
                 (:output
                  (cond
-                  ((null output)) ; No output.
-                  ((eq output t)) ; String output (file path).
-                  ((stringp output)
-                   (when (string-equal output "")
+                  ((null v)) ; No output.
+                  ((eq v t)) ; String output (file path).
+                  ((stringp v)
+                   (when (string-equal v "")
                      (error "Empty string used as file-path")))
-                  ((bufferp output)
-                   (unless (buffer-live-p output)
-                     (error "Input buffer is invalid %S" output)))
+                  ((bufferp v)
+                   (unless (buffer-live-p v)
+                     (error "Output buffer is invalid %S" v)))
 
                   (t
                    (error "Output expected a buffer, string or nil")))
@@ -372,7 +371,7 @@ Additional keyword arguments may also be passed in.
                   (erase-buffer))))
 
               (process-send-eof proc)
-              (while (not sentinel-called)
+              (while (null sentinel-called)
                 (accept-process-output))
 
               ;; Check exit-code.
@@ -447,22 +446,21 @@ Optional keywords in KEYWORDS.
                 ((null v)) ; Ignore.
                 ((floatp v))
                 (t
-                 (error "Argument :idle expected a float, not a %S" (type-of v))))
+                 (error "Idle expected a float, not a %S" (type-of v))))
                (setq idle v))
 
               (:jobs
                (cond
-                ((null v)) ; Ignore.
-                ((eq v t)) ; String output (result vector)
-                ((symbolp v))
+                ((null v)) ; Ignore (detect).
+                ((integerp v)) ; Integer jobs.
                 (t
-                 (error "Output expected nil, t or a symbol")))
+                 (error "Jobs expected nil or an integer")))
                (setq jobs v))
 
               (:output
                (cond
                 ((null v)) ; Ignore.
-                ((eq v t)) ; String output (result vector)
+                ((eq v t)) ; True for string output.
                 ((functionp v)
                  (setq output-is-fn t))
                 (t
@@ -472,8 +470,7 @@ Optional keywords in KEYWORDS.
               (:progress-message
                (cond
                 ((null v)) ; Ignore.
-                ((stringp v)
-                 (setq output-is-fn t))
+                ((stringp v)) ; String message.
                 (t
                  (error "Progress-message expected nil or string, not a %S" (type-of v))))
                (setq progress-message v))
@@ -488,14 +485,13 @@ Optional keywords in KEYWORDS.
     (unless idle
       (setq idle 0.01))
 
-    ;; TODO: detect.
     (unless jobs
       (setq jobs
             (cond
-             ((fboundp 'num-processors) ; Emacs 29+
-              (* 2 (num-processors)))
+             (diff-ansi-multiprocess-jobs
+              diff-ansi-multiprocess-jobs)
              (t
-              (or diff-ansi-multiprocess-jobs 64)))))
+              (* 2 (num-processors))))))
 
     (let* ((args-len (length args))
            (results (make-vector args-len nil))
@@ -555,7 +551,7 @@ Optional keywords in KEYWORDS.
 
       ;; Setup indices to represent jobs that aren't complete.
       (let ((i args-len))
-        (while (not (zerop i))
+        (while (null (zerop i))
           (push (decf i) jobs-pending)))
 
       ;; Main loop to complete
@@ -601,12 +597,10 @@ Optional keywords in KEYWORDS.
   "Return the HEX color from FACE and ATTR."
   (declare (important-return-value t))
   (apply #'format
-         (cons
-          "#%02x%02x%02x"
-          (mapcar
-           ;; Shift by -8 to map the value returned by `color values':
-           ;; 0..65535 to 0..255 for `#RRGGBB` string formatting.
-           (lambda (n) (ash n -8)) (color-values (face-attribute face attr))))))
+         "#%02x%02x%02x"
+         ;; Shift by -8 to map the value returned by `color values':
+         ;; 0..65535 to 0..255 for `#RRGGBB` string formatting.
+         (mapcar (lambda (n) (ash n -8)) (color-values (face-attribute face attr)))))
 
 
 ;; ---------------------------------------------------------------------------
@@ -693,7 +687,7 @@ Store the result in TARGET-BUF when non-nil."
 Argument BEG is only used to calculate the progress percentage."
   (declare (important-return-value nil))
   (cond
-   ((not (buffer-live-p buf))
+   ((null (buffer-live-p buf))
     ;; The buffer was closed (most likely by the user) while the timer was running,  cancel it.
     (cancel-timer timer))
    ((input-pending-p)
@@ -769,13 +763,12 @@ Store the result in TARGET-BUF when non-nil."
         (diff-ansi--call-process-pipe-chain diff-command :input diff-str :output (current-buffer))
         (setq end (point))
 
-        ;; Postpone activation until the timer can take it's self as an argument.
-        (diff-ansi--with-advice ((#'timer-activate :override (lambda (&rest _) nil)))
-          (setq diff-ansi--ansi-color-timer (run-at-time 0.0 0.001 nil))
-          (timer-set-function diff-ansi--ansi-color-timer #'diff-ansi-progressive-highlight-impl
-                              (list
-                               (current-buffer) beg (cons beg end) diff-ansi--ansi-color-timer)))
-        (timer-activate diff-ansi--ansi-color-timer)))))
+        (letrec ((timer
+                  (run-at-time 0.0 0.001
+                               (lambda (buf beg range)
+                                 (diff-ansi-progressive-highlight-impl buf beg range timer))
+                               (current-buffer) beg (cons beg end))))
+          (setq diff-ansi--ansi-color-timer timer))))))
 
 
 ;; ---------------------------------------------------------------------------
@@ -812,9 +805,9 @@ Store the result in TARGET-BUF when non-nil."
                    (emacs-eval-arg (diff-ansi--ansi-color-apply-on-region-with-bg-str black-color))
 
                    (point-prev (point))
-                   ;; Index (for unique names.
+                   ;; Index (for unique names).
                    (i 0))
-              (while (not (eobp))
+              (while (null (eobp))
                 (setq point-prev (point))
                 (goto-char (min (+ (point) diff-ansi-chunks-size) (point-max)))
                 (goto-char (pos-eol))
@@ -963,7 +956,7 @@ Return the buffer used to write data into on success."
 
     (let ((method diff-ansi-method))
 
-      ;; Fallback to immediate when the task wont be split.
+      ;; Fallback to immediate when the task won't be split.
       (when (<= (- end beg) diff-ansi-chunks-size)
         (setq method 'immediate))
 
@@ -979,7 +972,7 @@ Return the buffer used to write data into on success."
 
     ;; Update end.
     (unless target-buf
-      (setq end (- (point-max) end-trailing-chars)))
+      (setq end (- (buffer-size) end-trailing-chars)))
 
     ;; This is only an approximation, as the diff views may not be exactly compatible.
     (when preserve-point
@@ -1001,7 +994,8 @@ Return the buffer used to write data into on success."
 
     (cond
      (create-buffer
-      (set-buffer-modified-p nil)
+      (with-current-buffer target-buf
+        (set-buffer-modified-p nil))
       target-buf)
      (t
       (current-buffer)))))
@@ -1017,8 +1011,9 @@ Return the buffer used to write data into on success."
 ;;;###autoload
 (define-minor-mode diff-ansi-mode
   "Enable `diff-ansi' integration."
-  :global
-  nil
+  :global nil
+  :lighter nil
+
   (cond
    (diff-ansi-mode
     (diff-ansi--enable))
